if ui and ui.exit then
    ui.exit()
end

ui = {}

ui.help = [[
UI module is partially based on my and partially on Syrk's scripts (that are based on others)
TODO: more history / sources
TODO: 720p layout
]]

ui.settings = {
    console_btn_inactive = [[ background: rgba(60%, 60%, 60%, 50%); border-radius: 7; ]],
    console_btn_active   = [[ background: rgba(70%, 30%, 30%, 90%); border-radius: 7; ]],
    console_btn_current  = [[ background: rgba(30%, 30%, 60%, 100%); border-radius: 7; ]],
    console_style        = [[ background: rgba(20%, 20%, 20%, 100%);]],
    magic_map_default    = true,
    demo_tabs            = false, -- create demo tabs
    rulers               = false, -- create resolution rulers
    name = "UI", -- information required by logger: name, debug_levels
    debug_levels = { other   = 10 },
}

ui.state = {
    width = 0, -- 0 - full, 1 - 1920x1080, 2 - 1366x768
}

ui.containers = {}
ui.gauges = {}

local c = ui.containers
local s = ui.state

-------------------- Tabbed window, with clickable buttons,

local TabbedPanelCnt = 0
local TabbedPanel = {
    button_vertical = true, -- false = horizontal
    button_visible = true, -- false == hidden
    button_style_active = ui.settings.console_btn_active,
    button_style_inactive = ui.settings.console_btn_inactive,
    button_style_current = ui.settings.console_btn_current,
    button_width = 100,
    button_spacing = 5,
    button_height = 20,
    tabs = {},
    tab_count = 0,
    buttons = {},
}

ui.TabbedPanel = TabbedPanel

function TabbedPanel:new(o, layout)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    -- initial layout
    o.layout = layout
    o.name = o.name or "TabbedPanel"..tostring(TabbedPanelCnt)
    TabbedPanelCnt = TabbedPanelCnt + 1
    return o
end

function TabbedPanel:getLayout()
    return self.layout
end

-- returns lua layout
function TabbedPanel:add(name, contents)
    if not name or self.tabs[name] then
        return
    end

    local tab

    if not contents then
        tab = Geyser.Container:new({ name = "ui."..self.name..".tab_"..name,
                                     x=0, y=0, width ="100%", height="100%"}, self.layout)
    else
        tab = contents
    end

    self.tabs[name] = tab
    lowerWindow(tab.name)

    local button

    if self.button_vertical then
        button = Geyser.Label:new({ name = "ui."..self.name..".tab_btn_"..name,
        x =-(self.button_width + self.button_spacing),
        y = self.button_spacing + (self.button_height + self.button_spacing) * self.tab_count,
        width=self.button_width,
        height=self.button_height,
        message="<center>"..name.."</center>",
        },
        self.layout);
    else
        button = Geyser.Label:new({ name = "ui."..self.name..".tab_btn_"..name,
        x =-(self.button_width + self.button_spacing) * (self.tab_count + 1),
        y = self.button_spacing,
        width=self.button_width,
        height=self.button_height,
        message="<center>"..name.."</center>",
        },
        self.layout);
    end
    self.buttons[name] = button
    raiseWindow(button.name)

    if not self.button_visible then
        button:hide()
    end
    button:setStyleSheet(self.button_style_inactive)
    button:setClickCallback("ui.TabbedPanel.switch", self, name)

    self.tab_count = self.tab_count + 1

    if self.tab_count == 1 then
        self:switch(name)
    else
        tab:hide()
    end
end

-- returns lua layout or whatever we put into tab
function TabbedPanel:get(name)
    return self.tabs[name]
end

function TabbedPanel:switch(name)
    if self.current_tab then
        if self.current_tab == name then
            return
        end

        self.tabs[self.current_tab]:hide()
        self.buttons[self.current_tab]:setStyleSheet(self.button_style_inactive)
        raiseWindow(self.buttons[self.current_tab].name)

    end

    self.current_tab = name

    self.tabs[self.current_tab]:show()
    lowerWindow(self.tabs[self.current_tab].name) -- TODO: to not overlap with labels?
    self.buttons[self.current_tab]:setStyleSheet(self.button_style_current)
    raiseWindow(self.buttons[self.current_tab].name)
end

-- set activity button
function TabbedPanel:activate(name)
    if self.current_tab and self.current_tab ~= name then
        self.buttons[name]:setStyleSheet(self.button_style_active)
    end
end

function TabbedPanel:setVisible(vis)
    if vis then
        self.layout:show()
    else
        self.layout:hide()
    end
end

function TabbedPanel:show()
    self:setVisible(true)
end

function TabbedPanel:hide()
    self:setVisible(false)
end


function ui.switchMap()
    if ui.maps.current_tab ~= "mapper" then
        ui.maps:switch("mapper")
    else
        ui.maps:switch("magic_map")
    end
end

function ui.consoleAdd(name)
    local full_name = "ui.consoles."..name

    if ui.consoles:get(name) ~= nil then
        return
    end
    ui.consoles:add(name, Geyser.MiniConsole:new({name=full_name, x=10, y=10, width=-10, height=-10}, ui.consoles:getLayout()))
    local console = ui.consoles:get(name)
    -- console
    console:setFontSize(10)
    setBackgroundColor(full_name, 10, 10, 10, 100)
    setTextFormat(full_name, 10, 10, 10, 255, 255, 255, false, false, false)
    setWindowWrap(full_name, 95)
end

-- Add something to named console
function ui.consoleEcho(name, content)
    local console = ui.consoles:get(name)

    if not console then
        return
    end

    console:echo(content)
    ui.consoles:activate(name)
end

function ui.consoleCecho(name, content)
    local console = ui.consoles:get(name)

    if not console then
        return
    end

    console:cecho(content)
    ui.consoles:activate(name)
end

-- Init containers and frames
function ui.initContainers()
    local side_width, map_width, float_width
    local mainWidth, mainHeight = getMainWindowSize()

    float_width = 300
    map_height = 768
    extra_height = 100
    if s.width == 0 then
        map_width = 500
        side_width = float_width + map_width
    elseif s.width == 1 then
        map_width = 500
        side_width = float_width + map_width
    else
        map_width = 500
        side_width = 500
    end

    -- resize main console
    setBorderLeft(side_width + 5)
--    if s.width == 0 then
        setBorderRight(0)
        setBorderBottom(20)
 --   elseif s.width == 1 then
   --     setBorderRight(mainWidth - 1920)
   --     setBorderBottom(mainHeight - 1080 + 20)
   -- else
    --    setBorderRight(mainWidth - 1366)
    --    setBorderBottom(mainHeight - 768 + 20)
   -- end
    setBorderColor(50, 50, 50)

    c.top = Geyser.Container:new({ name = "ui.top.panel", x=0, y=0, width = "100%", height="100%"})
    -- left side of main console
    c.left_panel = Geyser.Container:new({ name = "ui.left.panel", x=0, y=0, width = side_width, height="100%"}, c.top)
    -- below main console
    c.bottom_panel = Geyser.Container:new({ name = "ui.bottom.panel", x=side_width, y=-20, width =-20, height=20}, c.top)

    -- at main console
    c.main_panel = Geyser.Container:new({ name = "ui.main.panel", x=side_width, y=0, width=mainWidth - side_width, height="100%"}, c.top)

    -- floater infopanel
    c.float_panel =  Geyser.Label:new({ name = "ui.float.panel", x="-35%", y=50, width =float_width, height=map_height}, c.main_panel)
    c.float_panel:setStyleSheet("border: 1px solid blue;")

    -- fixed infopanel
    c.fixed_panel = Geyser.Label:new({ name = "ui.fixed.panel", x=0, y=0, width = float_width, height=map_height}, c.left_panel)
    c.fixed_panel:setStyleSheet("border: 1px solid blue;")

    -- map(s) panel
    c.map_panel = Geyser.Container:new({ name = "ui.map.panel", x=side_width - map_width+1, y=0, width = map_width, height=map_height}, c.left_panel)

    c.extra_panel = Geyser.Label:new({ name = "ui.extra.panel", x=0, y=map_height+1, width = side_width, height=extra_height}, c.left_panel)
    c.extra_panel:setStyleSheet("border: 1px solid blue;")

    if s.width == 1 then
        c.float_panel:hide()
    end

    if s.width == 2 then
        c.fixed_panel:hide()
        c.float_panel:hide()
        c.extra_panel:hide()
    end

    if s.width ~= 2 then
        c.consoles_panel = Geyser.Label:new({ name = "ui.consoles.panel", x=0, y=map_height + extra_height+1, width = side_width, height=-1}, c.left_panel)
    else
        c.consoles_panel = Geyser.Label:new({ name = "ui.consoles.panel", x=side_width, y=10, width = -20, height="20%"}, c.top)
    end
    c.consoles_panel:setStyleSheet("border: 1px solid yellow;")

    -- Now tabbed containers
    ui.consoles = TabbedPanel:new({}, c.consoles_panel)
    ui.maps = TabbedPanel:new({button_visible = true}, c.map_panel)
    if s.width == 0 then
        ui.info = TabbedPanel:new({button_vertical = false, button_width = 50}, c.fixed_panel)
        ui.float = TabbedPanel:new({button_vertical = false, button_width = 50}, c.float_panel)
        ui.extra = TabbedPanel:new({button_vertical = false, button_width = 50}, c.extra_panel)
    elseif s.width == 1 then
        ui.info = TabbedPanel:new({button_vertical = false, button_width = 50}, c.fixed_panel)
        ui.float = ui.info
        ui.extra = TabbedPanel:new({button_vertical = false, button_width = 50}, c.extra_panel)
    elseif s.width == 2 then
        ui.info = ui.maps
        ui.float = ui.maps
        ui.extra = ui.consoles
    end

    c.gauges = Geyser.HBox:new({ name = "ui.gauges.layout", x=0, y=0, width = "100%", height="100%"}, c.bottom_panel)

end

function ui.initMaps()
  -- magic map - fake mapper

    ui.maps:add("magic_map", Geyser.MiniConsole:new({name = "ui.magic_map", x = 5, y = 5,
                                           width = -5, height = -5,
                                           color = "black"}, ui.maps:getLayout()));
    setMiniConsoleFontSize("ui.magic_map", 7)
    setWindowWrap("ui.magic_map", 120)

    -- real mapper
    ui.maps:add("mapper", Geyser.Mapper:new({name = "ui.mapper", x = 5, y = 5, width = -5, height = -5}, ui.maps:getLayout()))

    -- Switch to magic map
    if ui.settings.magic_map_default == false then
        ui.switchMap()
    end
end

function ui.addGauge(name, style_front, style_back)
    local g = ui.gauges
    g[name] = Geyser.Gauge:new({  name="ui.gauges."..name,  width="60px", height="100%", }, c.gauges)
    g[name].front:setStyleSheet(style_front)
    g[name].back:setStyleSheet(style_back)
end

function ui.setGauge(name, value, max, text)
    local g = ui.gauges

    g[name]:setValue(value, max, text)
end

function ui.hasGauge(name)
    local g = ui.gauges

    return g[name]
end

function ui.initGauges()
    ui.addGauge("health",
        [[background-color: rgba(100%, 0%, 0%, 100%); border-radius: 7; padding: 3px; ]],
        [[background-color: rgba(60%, 0%, 0%, 100%); border-radius: 7; padding: 3px; ]])

    ui.addGauge("mana",
        [[background-color: rgba(0%, 0%, 100%, 100%); border-radius: 7; padding: 3px; ]],
        [[background-color: rgba(0%, 0%, 60%, 100%); border-radius: 7; padding: 3px; ]])

    ui.addGauge("fatigue",
        [[background-color: rgba(0%, 100%, 0%, 100%); border-radius: 7; padding: 3px; ]],
        [[background-color: rgba(0%, 60%, 0%, 100%); border-radius: 7; padding: 3px; ]])

    ui.addGauge("food",
        [[background-color: rgb(181, 106, 1); border-radius: 7; padding: 3px; ]],
        [[background-color: rgb(130, 75, 0); border-radius: 7; padding: 3px; ]])

    ui.addGauge("drink",
        [[background-color: rgb(1, 166, 191); border-radius: 7; padding: 3px; ]],
        [[background-color: rgb(0, 101, 117); border-radius: 7; padding: 3px; ]])

    ui.addGauge("intoxicated",
        [[background-color: rgb(85, 2, 201); border-radius: 7; padding: 3px; ]],
        [[background-color: rgb(55, 2, 130); border-radius: 7; padding: 3px; ]])
end


-- tmp
function ui.screenFrames()
    c.frame_1080 = Geyser.Label:new({ name = "ui.frame_1080", x=0, y=0, width="1920", height="1080"},
        c.top);
    c.frame_1080:setStyleSheet("border: 1px dashed red;")
    c.frame_720 = Geyser.Label:new({ name = "ui.frame_720", x=0, y=0, width="1366", height="768"},
        c.top);
    c.frame_720:setStyleSheet("border: 1px dashed red;")
    lowerWindow("ui.frame_1080")
    lowerWindow("ui.frame_720")
    c.frame_1080:hide()
    c.frame_720:hide()
end

-- tmp, just for showoff
function ui.demoTabs()
    if ui.settings.rulers then
        ui.screenFrames()
    end

    if ui.settings.demo_tabs then
        ui.consoleAdd("chat")
        ui.info:add("info1", Geyser.MiniConsole:new({name="tmp_info1", x=10, y=10, width=-10, height=-10}, ui.info:getLayout()))
        ui.info:add("info2", Geyser.MiniConsole:new({name="tmp_info2", x=10, y=10, width=-10, height=-10}, ui.info:getLayout()))
        ui.float:add("float1", Geyser.MiniConsole:new({name="tmp_float1", x=10, y=10, width=-10, height=-10}, ui.float:getLayout()))
        ui.float:add("float2", Geyser.MiniConsole:new({name="tmp_float2", x=10, y=10, width=-10, height=-10}, ui.float:getLayout()))
        local l = Geyser.Label:new({ name = "tmp_char", x=5, y=5, width = -5, height=-5}, ui.extra:getLayout())
        l:echo("Placeholder for some char fun\n")
        ui.extra:add("char", l)
        local l = Geyser.Label:new({ name = "tmp_stat", x=5, y=5, width = -5, height=-5}, ui.extra:getLayout())
        ui.extra:add("stat", l)
        l:echo("Placeholder for uber stats\n")
        local l = Geyser.Label:new({ name = "tmp_loc", x=5, y=5, width = -5, height=-5}, ui.extra:getLayout())
        ui.extra:add("loc", l)
        l:echo("Placeholder for location info\n")
    end
end

-- 1440 layout, TODO: others
function ui.init()
    mainWidth, mainHeight = getMainWindowSize()

    -- TODO: autodetect
    s.width = 0
    if mainWidth <= 2400 then
        s.width = 1
    end
    if mainWidth <= 1920 then
        s.width = 2
    end

    --s.width = 2

    ui.initContainers()
    ui.demoTabs()
    ui.initMaps()
    ui.initGauges()
    -- Add mail icon
    ui.mail = Geyser.Label:new({ name = "ui.mail", x=0, y=0, width="60px", height="100%"}, c.gauges);
    ui.mail:setStyleSheet("border: 1px solid white;")
    ui.mail:echo("mail unknown")
end

function ui.exit()
end

if ui then
    genLoad("ui-user", true)
    if ui.init then
        ui.init()
    end
end
