-- exit if it was loaded, initialize AT END OF FILE
if inv and inv.exit then
    inv.exit()
end

inv = {}

-- fixed thing that is not removed
inv_static = inv_static or {
    locations = {} -- map item:location or item:{location}, that way we can figure out where item belongs
}

inv.settings = {
    name = "I", -- information required by logger: name, debug_levels
    debug_levels = {  -- 0 - print no messages (but critical errors), 10 - print everything
    other   = 10 },
}

inv.ui = {}

inv.state = {
    held = {},
    wielded = {},
    pocket = {},
    value = {},
    worn= {},
    worn_nowhere = {}, -- worn ,but don't know where
    container_transaction = false, -- are we in middle of cntainer transaction (i.e. there is "close"?
    container_transaction_opened = {}, -- containers that are opened in current transaction
    old_inv_string = "", -- previos inventory string
}

local settings = inv.settings
local state = inv.state
local static = inv_static

-- slot abbreviations - location to listed location
settings.slot_abbrev = {
    ["left ring finger"] = "FingerL",
    ["right ring finger"] = "FingerR",
    ["left shoulder"] = "ShouldL",
    ["right shoulder"] = "ShouldR",
    ["left wrist"] = "WristL",
    ["right wrist"] = "WristR",
}

-- list of worn slots in order they appear
settings.slot_list = {
    "head",
    "ear",
    "neck",
    "",
    "chest",
    "arms",
    "body",
    "shoulders",
    "legs",
    "",
    "hands",
    "left shoulder",
    "right shoulder",
    "left wrist",
    "right wrist",
    "left ring finger",
    "right ring finger",
    "waist",
    "feet",
    ""
}

-- internal state tracking
function inv._removeGear(container, what)
        -- just brute force worn items
    for loc, wat in pairs(container) do
        if wat == what then
            container[loc] = nil
            inv.updateUi()
            return
        end
    end
end

function inv.worn(event, what, where)
    if not where then
        -- TODO: we got the same event if we wear item twice, kill duplicates
        if not table.index_of(state.worn_nowhere, what) then
            table.insert(state.worn_nowhere, what)
        else
            log.info(inv, nil, "Possible duplicate item: %s\n", what)
        end
    else
        state.worn[where] = what
        static.locations[what] = where
    end
    inv.updateUi()
end

function inv.removed(event, what)
    local where = static.locations[what]

    if (state.worn[where]) then
        state.worn[where] = nil
        inv.updateUi()
        return
    end

    -- try to look it up at "nowhere" list
    local idx = table.index_of(state.worn_nowhere, what)
    if idx then
        table.remove(state.worn_nowhere, idx)
        inv.updateUi()
        return
    end

    inv._removeGear(state.worn, what)
end

function inv.held(event, what, where)
    state.held[where] = what
    inv.updateUi()
end

function inv.released(event, what)
    inv._removeGear(state.held, what)
end

function inv.wielded(event, what, where)
    state.wielded[where] = what
    inv.updateUi()
end

function inv.unwielded(event, what)
    inv._removeGear(state.wielded, what)
end

----------------------------------------------------------- external interface
function inv.init()
    log.info(inv, nil, "Initializing warehouseman module...\n")

    inv.ui = Geyser.MiniConsole:new({name="inv.inv_console", x=10, y=10, width=-10, height=-10}, ui.info:getLayout())
    ui.info:add("inv", inv.ui)
    inv.ui:setFontSize(8)
    setBackgroundColor(inv.ui.name, 10, 10, 10, 100)
    setTextFormat(inv.ui.name, 10, 10, 10, 255, 255, 255, false, false, false)
    setWindowWrap(inv.ui.name, 40)
    inv.ui:clear()

    registerAnonymousEventHandler("obs_item_worn", "inv.worn")
    registerAnonymousEventHandler("obs_item_removed", "inv.removed")
    registerAnonymousEventHandler("obs_item_held", "inv.held")
    registerAnonymousEventHandler("obs_item_released", "inv.released")
    registerAnonymousEventHandler("obs_item_wielded", "inv.wielded")
    registerAnonymousEventHandler("obs_item_unwielded", "inv.unwielded")
end

function inv.updateUi()
    local inv_string=""
    local empty_inv="~~~~"
    local new_inv = {}
    local twohander = state.wielded["hands"] or state.held["hands"]

    table.insert(new_inv, "<green>Hands")
    if state.wielded["left hand"] then
        table.insert(new_inv, "<sea_green>     Left: <white>"..state.wielded["left hand"])
    elseif state.held["left hand"] then
        table.insert(new_inv, "<sea_green>     Left: <white>"..state.held["left hand"])
    elseif twohander then
        if state.wielded["hands"] then
            table.insert(new_inv, "<sea_green>     Left: <white>"..state.wielded["left hand"])
            table.insert(new_inv, "<dim_gray>    Right: (two handed)")
            tmp = true
        end
        if state.held["hands"] then
            table.insert(new_inv, "<sea_green>     Left: <white>"..state.held["left hand"])
            table.insert(new_inv, "<dim_gray>    Right: <dim_gray>(two handed)")
            tmp = true
        end
    else
        table.insert(new_inv, "<dim_gray>     Left: ".. empty_inv)
    end

    if state.wielded["right hand"] then
        table.insert(new_inv, "<sea_green>    Right: <white>"..state.wielded["right hand"])
    elseif state.wielded["right hand"] then
        table.insert(new_inv, "<sea_green>    Right: <white>"..state.held["right hand"])
    else
        table.insert(new_inv, "<dim_gray>    Right: "..empty_inv)
    end

    -- Now process ordinary slots
    table.insert(new_inv, "<green>Worn")
    for t, slot in ipairs(settings.slot_list) do
        if slot ~= "" then
            local item = state.worn[slot]

            local heading = settings.slot_abbrev[slot] or string.title(slot)
            if item then
                table.insert(new_inv, string.format("<sea_green>%9s: <white>%s", heading, item))
            else
                table.insert(new_inv, string.format("<dim_grey>%9s: %s", heading, empty_inv))
            end
        else
            table.insert(new_inv, "")
        end
    end

    if #state.worn_nowhere then
        table.insert(new_inv, string.format("<sea_green>Unknown locations\n<white>%s\n", table.concat(state.worn_nowhere, "\n")))
    end

    if #state.value > 0 then
        table.insert(new_inv, string.format("<gold>Valuables\n<white>%s", table.concat(state.value, "\n").."\n"))
    end
    if #state.pocket > 0 then
        table.insert(new_inv, string.format("<green>Pocket\n<white>%s", table.concat(state.pocket, "\n") .. "\n"))
    end

    -- join and print
    inv_string = table.concat(new_inv, "\n")

    if inv_string ~= state.old_inv_string then
        state.old_inv_string = inv_string
        inv.ui:clear()
        inv.ui:cecho(inv_string)
    end
end

function inv.exit()
    log.info(inv, nil, "Closing warehouseman module...\n")
    if inv.state.container_transaction then
        mud.sendCmd("close all")
        killTimer(inv.state.container_transaction_timer)
    end
end

-- Resolves letter (or full) name into mud name
function invContainerName(what)
    if inv.containerName[what] then
        what = inv.containerName[what]
    elseif inv.containerKey[what] then
        what = inv.containerKey[what]
    end
    return what
end

function invContainerOf(what)
    local container = inv.containerOf[what]
    if not container then
        container = inv.containerOfEx[what]
    end

    if container then
        return invContainerName(container)
    end
    return nil
end

function invList(what)
    local c = inv.containerKey[what]
    local n = inv.containerName[what] or c or what
    if c and (c == "jar" or c == "qarraba") then
        mud.sendCmd("qlist")
    else
        mud.sendCmd("open "..n)
        mud.sendCmd("exa "..n)
        mud.sendCmd("close "..n)
    end
end

function invStartTransaction(time)
    if not time then
        time = 0
    end

    if not inv.state.container_transaction then
        log.dbg(inv, nil, "Starting inventory transaction\n")
        inv.state.container_transaction = true
        inv.state.container_transaction_opened = {}
    end

    if inv.state.container_transaction_timer then
        killTimer(inv.state.container_transaction_timer)
    end
    -- Just for safety
    inv.state.container_transaction_timer = tempTimer(time, [[invEndTransaction()]])
end

function invEndTransaction()
    log.dbg(inv, nil, "Ending inventory transaction\n")
    mud.sendCmd("close all")
    inv.state.container_transaction = false
    inv.state.container_transaction_opened = {}
end

function invStoreTo(what, where)
 mud.sendCmd("open "..where)
 mud.sendCmd("put "..what.." in "..where)
 mud.sendCmd("close "..where)
end

function invRetrieveFrom(what, where)
 mud.sendCmd("open "..where)
 mud.sendCmd("get "..what.." from "..where)
 mud.sendCmd("close "..where)
end

function invRetrieveMany(what, where, count)
 mud.sendCmd("open "..where)
 for i=1, count, 1 do
  mud.sendCmd("get "..what.." from "..where)
 end
 mud.sendCmd("close "..where)
end

function invStore(what)
 local c = invContainerOf(what)
 if not c then
  log.err(inv, nil, "AutoStore not implemented for "..what.."\n")
  invStoreTo(what, "backpack")
 else
  invStoreTo(what, c)
 end
end

function invRetrieve(what)
 local c = invContainerOf(what)
 if not c then
  log.err(inv, nil, "AutoRetrieve not implemented for "..what.."\n")
  invRetrieveFrom(what, "backpack")
 else
  invRetrieveFrom(what, c)
 end
end

-- Initialize
if inv and inv.init then
    -- Load submodules
    genLoad("warehouseman-config")
    genLoad("warehouseman-user", true)
    inv.init()
end
